@@ -26,7 +26,7 @@ use strict;
 use Getopt::Long qw(:config pass_through no_auto_abbrev);
 
 my $app_name     = 'colordiff';
-my $version      = '1.0.20';
+my $version      = '1.0.21';
 my $author       = 'Dave Ewart';
 my $author_email = 'davee@sungate.co.uk';
 my $app_www      = 'http://www.colordiff.org/';
@@ -75,7 +75,8 @@ my $color_patch      = undef;
 # Locations for personal and system-wide colour configurations
 my $HOME = (grep { defined && length }
         @ENV{qw/HOME USERPROFILE/})[0] || '';
-my $etcdir = '/etc';
+my $etcdir = (grep { defined && length && -d }
+	'/etc', $ENV{ALLUSERSPROFILE})[0] || '/etc';
 my ($setting, $value);
 my @config_files = ("$etcdir/colordiffrc");
 my $USER_CONFIG_DIR = (grep { defined && length }
@@ -179,6 +180,8 @@ my $enable_verifymode;
 my $specified_difftype;
 my $enable_fakeexitcode;
 my $color_term_output_only = "no";
+my $cmd_banner;
+my $cmd_help;
 
 # Command-line options may over-ride config files
 GetOptions(
@@ -189,9 +192,25 @@ GetOptions(
     "difftype=s" => \$specified_difftype,
     "color=s" => \$cmd_color_mode,
     "color-patches=s" => \$cmd_color_patch,
-    "color-term-output-only=s" => \$color_term_output_only
+    "color-term-output-only=s" => \$color_term_output_only,
+    "help" => \$cmd_help,
+    "banner!" => \$cmd_banner
 );
 
+if (defined $cmd_help) {
+    print "colordiff:\n";
+    print "    --help                   : Displays this help\n";
+    print "    --color=(yes|no)         : Force (or suppress) display of colours in output\n";
+    print "    --color=patches=(yes|no) : Force (or suppress) inclusion of colour codes in patch output\n";
+    print "    --color-term-output-only : Force (or suppress) colour to only appear in terminal output\n";
+    print "    --difftype=DIFFTYPE      : Force difftype detection to specified format\n";
+    print "    --(no)banner             : Show (or suppress) the colordiff banner\n";
+    print "\n";
+    print "    DIFFTYPE is usually auto-detected, but can be set to:\n";
+    print "          diff, diffc, diffu, diffy, debdiff or wdiff\n";
+    exit(0);
+}
+
 $_ = $specified_difftype;
 if (defined $_ and not /^diff[cuy]?|(deb|w)diff$/) {
     print STDERR "Invalid --difftype value\n";
@@ -218,7 +237,9 @@ foreach $config_file (@config_files) {
                 next;
             }
             if ($setting eq 'banner') {
-                if ($value eq 'no') {
+                if ($value eq 'yes') {
+                    $show_banner = 1;
+                } elsif ($value eq 'no') {
                     $show_banner = 0;
                 }
                 next;
@@ -364,6 +385,16 @@ if ( (!$color_mode) ||
     $colour{off} = '';
 }
 
+# If command-line option refers to banner, honour this ahead of config files
+if (defined $cmd_banner) {
+    if ($cmd_banner) {
+        $show_banner = 1;
+    }
+    if (!$cmd_banner) {
+        $show_banner = 0;
+    }
+}
+
 # Disable output buffering. This allows "producer | colordiff | less" to output
 # earlier without having to wait for 'producer' to finish.
 select STDOUT;
@@ -559,14 +590,14 @@ while (defined( $_ = @inputstream ? shift @inputstream : ($lastline and <$inputh
             $inside_file_old = 1;
         }
         elsif (/^\*\*\* /) {
-            print "$file_old";
+            print "$diff_file";
         }
         elsif (/^--- [0-9]+,[0-9]+/) {
             print "$diff_file";
             $inside_file_old = 0;
         }
         elsif (/^--- /) {
-            print "$file_new";
+            print "$diff_file";
         }
         elsif (/^!/) {
             if ($inside_file_old == 1) {
@@ -584,11 +615,8 @@ while (defined( $_ = @inputstream ? shift @inputstream : ($lastline and <$inputh
         }
     }
     elsif ($diff_type eq 'diffu') {
-        if (/^--- /) {
-            print "$file_old";
-        }
-        elsif (/^\+\+\+ /) {
-            print "$file_new";
+        if (/^(---|\+\+\+) /) {
+            print "$diff_file";
         }
         elsif (/^([-\+ ]{$count_marks})/) {
             my $diff_marks = $1;
